/***************************************************************************

  joystick.c

  Joystick devices component

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __JOYSTICK_C

#include "gambas.h"
#include "main.h"

#include "joystick.h"

#include <stdio.h>

// events
DECLARE_EVENT(EVENT_ButtonPress);
DECLARE_EVENT(EVENT_ButtonRelease);
DECLARE_EVENT(EVENT_AxisMove);
DECLARE_EVENT(EVENT_Error);

void Cjoystick_callback(int fd, int type, long param)
{
	void *_object = (void *) param;

	while (1)
	{
		joydev_getevents(&JOYSTICK, &JOYEVENT);

		switch(JOYEVENT.type)
		{
			case NO_EVENT:
				return;
			case ERROR:
				GB.Watch(JOYSTICK.fd, GB_WATCH_NONE, 0, 0);
				joydev_closedevice(&JOYSTICK);
				GB.Raise(THIS, EVENT_Error, 0);
				return;
			case JOY_BUTTON:
				if (JOYEVENT.value == BTN_PRESSED)
					GB.Raise(THIS, EVENT_ButtonPress, 0);
				else
					GB.Raise(THIS, EVENT_ButtonRelease, 0);
				break;
			case JOY_AXIS:
				GB.Raise(THIS, EVENT_AxisMove, 0);
			default:
				break;
		}
	}
}

BEGIN_METHOD(CJOYSTICK_open, GB_STRING device)

	/* Prevent a previously opened device */
	if (JOYSTICK.fd)
	{
		GB.Watch(JOYSTICK.fd, GB_WATCH_NONE, 0, 0);
		joydev_closedevice(&JOYSTICK);
	}

	if ((joydev_opendevice(STRING(device), &JOYSTICK)) < 0)
	{
		GB.Error("Failed to open &1", VARG(device));
		return;
	}

	GB.Watch(JOYSTICK.fd, GB_WATCH_READ, Cjoystick_callback, (long)THIS);

END_METHOD

BEGIN_METHOD_VOID(CJOYSTICK_free)

	if (JOYSTICK.fd)
	{
		GB.Watch(JOYSTICK.fd, GB_WATCH_NONE, 0, 0);
		joydev_closedevice(&JOYSTICK);
	}

END_METHOD

BEGIN_PROPERTY(CJOYSTICK_name)

	GB.ReturnNewZeroString(JOYSTICK.name);

END_PROPERTY

BEGIN_PROPERTY(CJOYSTICK_path)

	GB.ReturnNewZeroString(JOYSTICK.path);

END_PROPERTY

BEGIN_PROPERTY(CJOYSTICK_buttons)

	GB.ReturnInteger(JOYSTICK.buttons);

END_PROPERTY

BEGIN_PROPERTY(CJOYSTICK_axises)

	GB.ReturnInteger(JOYSTICK.axises);

END_PROPERTY

BEGIN_PROPERTY(CJOYSTICK_button)

	GB.ReturnInteger(JOYEVENT.number);

END_PROPERTY

BEGIN_PROPERTY(CJOYSTICK_axis)

	GB.ReturnInteger(JOYEVENT.number);

END_PROPERTY

BEGIN_PROPERTY(CJOYSTICK_axisvalue)

	GB.ReturnInteger(JOYEVENT.value);

END_PROPERTY

/***************************************************************************/

GB_DESC CJoystickDesc[] =
{
  GB_DECLARE("Joystick", sizeof(CJOYSTICK)),

  GB_METHOD("_free", NULL, CJOYSTICK_free, NULL),

  GB_METHOD("Open", NULL, CJOYSTICK_open, "[(Device)s]"),
  GB_METHOD("Close", NULL, CJOYSTICK_free, NULL),

  GB_PROPERTY_READ("Name", "s", CJOYSTICK_name),
  GB_PROPERTY_READ("Path", "s", CJOYSTICK_path),
  GB_PROPERTY_READ("Buttons", "i", CJOYSTICK_buttons),
  GB_PROPERTY_READ("Axises", "i", CJOYSTICK_axises),

  GB_PROPERTY_READ("Button", "i", CJOYSTICK_button),
  GB_PROPERTY_READ("Axis", "i", CJOYSTICK_axis),
  GB_PROPERTY_READ("AxisValue", "i", CJOYSTICK_axisvalue),

  GB_EVENT("ButtonPress", NULL, NULL, &EVENT_ButtonPress),
  GB_EVENT("ButtonRelease", NULL, NULL, &EVENT_ButtonRelease),
  GB_EVENT("AxisMove", NULL, NULL, &EVENT_AxisMove),
  GB_EVENT("Error", NULL, NULL, &EVENT_Error),

  GB_END_DECLARE
};
